"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.directoriesExtendsPageOptions = exports.prepareDirectories = void 0;
const utils_1 = require("@vuepress/utils");
const utils_2 = require("../utils");
/**
 * 开发时热更新
 */
const HMR_CODE = `
if (import.meta.webpackHot) {
  import.meta.webpackHot.accept()
  if (__VUE_HMR_RUNTIME__.updateBlogType) {
    __VUE_HMR_RUNTIME__.updateBlogType(directoryMap)
  }
}

if (import.meta.hot) {
  import.meta.hot.accept(({ directoryMap }) => {
    __VUE_HMR_RUNTIME__.updateBlogType(directoryMap)
  })
}
`;
const filterDir = (options, pages) => {
    const { directoryClassifier = [] } = options;
    return directoryClassifier.map(classfifier => {
        const map = {};
        map[classfifier.key] = [];
        const keys = [];
        for (const localPath in pages) {
            const classifierKey = pages[localPath].filter(page => page.frontmatter && page.frontmatter.classifier === classfifier.key).map(({ key }) => key);
            map[classfifier.key] = classifierKey;
            keys.push(...classifierKey);
        }
        return {
            map,
            keys
        };
    });
};
/**
 * 目录分配器
 * @param app
 * @param options
 * @param pages
 * @returns
 */
const prepareDirectories = (app, options, pages) => {
    return Promise.all(filterDir(options, pages)).then(async (result) => {
        if (app.env.isDebug)
            utils_2.logger.info('目录分配器', result);
        const finalMap = {};
        const keys = [];
        result.forEach(({ map, keys }) => {
            Object.assign(finalMap, map);
            keys.push(...keys);
        });
        await app.writeTemp(`blog/pageClassfier.js`, `export const directoryMap = ${JSON.stringify(finalMap)}\n${HMR_CODE}`);
        return keys;
    });
};
exports.prepareDirectories = prepareDirectories;
/**
 * 目录 extends page
 */
const directoriesExtendsPageOptions = (app, pageOptions, options) => {
    const { directoryClassifier = [] } = options;
    const appLayouts = Object.keys(app.layouts);
    const defaultAppLayout = appLayouts.length ? appLayouts[0] : 'Layout';
    directoryClassifier.forEach(classifier => {
        const { key, path: indexPath = `/${classifier.key}/`, dirname, layout = defaultAppLayout, itemLayout = defaultAppLayout, itemPermalink = '', frontmatter } = classifier;
        const { filePath } = pageOptions;
        const sourceDir = app.dir.source(dirname);
        if (filePath && filePath.startsWith(sourceDir)) {
            const reg = /[readme|index].md$/gi;
            const isIndex = reg.test(filePath);
            pageOptions.frontmatter = pageOptions.frontmatter ?? {};
            pageOptions.frontmatter = {
                ...pageOptions.frontmatter,
                layout: isIndex ? layout : itemLayout,
                ...frontmatter,
                classifier: key
            };
            const filePathRelative = filePath.replace(sourceDir + '/', '');
            const pathDir = utils_1.path.dirname(filePathRelative);
            if (isIndex) {
                pageOptions.path = indexPath + pathDir;
            }
            else {
                pageOptions.frontmatter.permalinkPattern = itemPermalink ? itemPermalink : `${indexPath}${pathDir}/:slug.html`;
            }
            pageOptions.frontmatter.indexPath = indexPath;
            pageOptions.frontmatter.dirname = dirname;
        }
    });
};
exports.directoriesExtendsPageOptions = directoriesExtendsPageOptions;
//# sourceMappingURL=prepareDirectories.js.map