"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.blogPlugin = void 0;
/**
 * blog plugin
 */
const utils_1 = require("./utils");
const classifier_1 = require("./classifier");
const chokidar_1 = require("chokidar");
const core_1 = require("@vuepress/core");
const blogPlugin = (options) => (app) => {
    const { getInfo = () => ({}), metaScope = '_blog' } = options;
    let generatePageKeys = [];
    if (app.env.isDebug)
        utils_1.logger.info(`Options: ${options.toString()}`);
    return {
        name: '@mr-huang/vuepress-plugin-blog',
        define: () => ({
            BLOG_META_SCOPE: metaScope,
        }),
        extendsPage(page) {
            page.routeMeta = {
                ...(metaScope === '' ? getInfo(page) : { [metaScope]: getInfo(page) }),
                ...page.routeMeta,
            };
        },
        // 为页面增强
        extendsPageOptions: (pageOptions) => {
            (0, classifier_1.directoriesExtendsPageOptions)(app, pageOptions, options);
        },
        // 初始化时创建页面
        onInitialized(app) {
            const pages = (0, utils_1.filterPages)(options, app);
            return Promise.all([
                (0, classifier_1.preparePageType)(app, options, pages, true).then(keys => {
                    generatePageKeys.push(...keys);
                }),
                (0, classifier_1.prepareDirectories)(app, options, pages).then(keys => {
                    generatePageKeys.push(...keys);
                }),
                (0, classifier_1.prepareFrontmatter)(app, options, pages, true).then(keys => {
                    generatePageKeys.push(...keys);
                })
            ]).then(() => {
                // console.log(app.pages)
            });
        },
        onWatched: (app, watchers) => {
            if (!options.hotReload)
                return;
            const pageDataWatcher = (0, chokidar_1.watch)('page/**/*.js', {
                cwd: app.dir.temp(),
                ignoreInitial: true
            });
            const updateBlogData = () => {
                const newGeneratedPageKeys = [];
                const pageMap = (0, utils_1.filterPages)(options, app);
                return Promise.all([
                    (0, classifier_1.prepareFrontmatter)(app, options, pageMap).then((pageKeys) => {
                        newGeneratedPageKeys.push(...pageKeys);
                    }),
                    (0, classifier_1.preparePageType)(app, options, pageMap).then((pageKeys) => {
                        newGeneratedPageKeys.push(...pageKeys);
                    }),
                    (0, classifier_1.prepareDirectories)(app, options, pageMap).then(pageKeys => {
                        newGeneratedPageKeys.push(...pageKeys);
                    })
                ]).then(async () => {
                    const pagestoBeRemoved = generatePageKeys.filter((key) => !newGeneratedPageKeys.includes(key));
                    const pagestoBeAdded = newGeneratedPageKeys.filter((key) => !generatePageKeys.includes(key));
                    if (pagestoBeAdded.length) {
                        if (app.env.isDebug)
                            utils_1.logger.info(`New pages detected: ${pagestoBeAdded.toString()}`);
                        // prepare page files
                        await Promise.all(pagestoBeAdded.map(async (pageKey) => {
                            await (0, core_1.preparePageComponent)(app, 
                            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                            app.pages.find(({ key }) => key === pageKey));
                            await (0, core_1.preparePageData)(app, 
                            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                            app.pages.find(({ key }) => key === pageKey));
                        }));
                    }
                    // remove pages
                    if (pagestoBeRemoved.length) {
                        if (app.env.isDebug)
                            utils_1.logger.info(`Removing following pages: ${pagestoBeRemoved.toString()}`);
                        pagestoBeRemoved.forEach((pageKey) => {
                            app.pages.splice(app.pages.findIndex(({ key }) => key === pageKey), 1);
                        });
                    }
                    // prepare pages entry
                    if (pagestoBeRemoved.length || pagestoBeAdded.length) {
                        await (0, core_1.preparePagesComponents)(app);
                        await (0, core_1.preparePagesData)(app);
                        await (0, core_1.preparePagesRoutes)(app);
                    }
                    generatePageKeys = newGeneratedPageKeys;
                    if (app.env.isDebug)
                        utils_1.logger.info("temp file updated");
                });
            };
            pageDataWatcher.on('add', () => {
                updateBlogData();
            });
            pageDataWatcher.on('change', () => {
                updateBlogData();
            });
            pageDataWatcher.on('unlink', () => {
                updateBlogData();
            });
            watchers.push(pageDataWatcher);
        }
    };
};
exports.blogPlugin = blogPlugin;
//# sourceMappingURL=plugin.js.map